# coding: utf-8
"""
 Die Klasse Grafikzeichner -- Version 2.0:
   Eine Erweiterung der Stifte und Mäusebibliothek
   für die Python-Portierung von Ingo Linkweiler
   mit kleinen eigenen Modifikationen vom 24.09.2008.

   Mit dieser Erweiterung können Objekte des
   Vektorgrafikpaketes PyObjVG gezeichnet werden.
   
   AUTOMATIKMODUS:
   Eine Verbindung zwischen Grafikzeichner und Vektorgrafik
   ermöglicht es, Änderungen von Attributen bei den
   beteiligten Grafikobjekten direkt auf dem Bildschirm
   sichtbar zu machen (Eine Neuzeichnung der Grafik muss
   nicht manuell angestoßen werden. Andere Objekte können
   jedoch nicht auf dem Bildschirm dargestellt werden, da vor
   jedem Neuzeichnen, der Bildschirm automatisch gelöscht wird).
   
   Autor: Matthias Heming <matthias@familie-heming.de>
   
   Dieses Klasse wird unter der Creative Commons by-nc-sa
   veroeffentlich, naehere Informationen dazu unter  
   http://creativecommons.org/licenses/by-nc-sa/3.0/de/.
   
   Versionsgeschichte:
     Version 1.0 vom 24. September 2008
     Version 2.0 vom 15. Januar 2009
     - Ueberarbeitung zur Python-New-Style-Klasse
     - Anpassung an Verbindungskonzept zu PyObjVG Version 2.0
"""

from pyobjvg import *

class Grafikzeichner(object):
    """
    Der Grafikzeichner kennt -- analog zu der Klasse Stift -- einen Bildschirm.
    Auf diesen Bildschirm kann ein Grafikzeichner Objekte des Types "Vektorgraphik"
    zeichnen.
    Ein Grafikzeichner kann ausserdem mit genau einer Vektorgrafik direkt verbunden werden.
    Wird der Automatik-Modus aktiviert, so wird bei Aenderungssignalen der
    Vektorgrafik eine Neuzeichnung ausgeloest.
    """
    def __init__(self, bildschirm=None):
        """
        Initialisierung des Grafikzeichners.
        Als Parameter muss der zugehörende Bildschirm angegeben werden.
        """
        if bildschirm != None:
          self.__bildschirm=bildschirm
        else:
          raise Exception("Es ist kein Bildschirm verfuegbar.")
        
        self.__ZugeordneteGrafik = None
        # keine Vektorgrafik verbunden
        self.__automatikmodus = False
        # es werden keine Aenderungssignale verarbeitet
        self.__freigegeben = True
    
    def gibFrei(self):
        """
        Loest eine eventuell vorhandene Verbindung zu einer Vektorgrafik,
        setzt die interne Bildschirmreferenz auf 'None' und deaktiviert
        internes Freigegeben-Flag, so dass nachfolgende Aufrufe
        eine Ausnahme erzeugen.
        """
        if self.__ZugeordneteGrafik is not None:
            self.objektTrennen(self.__ZugeordneteGrafik)
        self.__bildschirm = None
        self.__freigegeben = False
    
    def setzeAutomatikModus(self,status):
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        if status:
            self.__automatikmodus = True
        else:
            self.__automatikmodus = False
    
    def gibAutomatikModusStatus(self):
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        return self.__automatikmodus
    
    automatikModus = property(fget=gibAutomatikModusStatus,fset=setzeAutomatikModus,fdel=None,doc=\
            """
            Status-Flag des automatischen Zeichenmodus:
            True:  Aenderungssignale vom verbundenen Objekt fuehren zur
                   automatisierten Neuzeichnung der Vektorgrafik auf dem
                   Bildschirm. Dazu wird der Bildschirm vorher geloescht.
            False: Keine automatische Neuzeichnung.
            """)
    def objektVerbinden(self,neuesobjekt):
        """
        Es wird das uebergebene Objekt mit dem Zeichner verbunden.
        
        Falls die Methode nicht ueber das uebergebene Objekt
        aufgerufen wurde, wird auch aus Sicht des Objekts
        die Verbindung hergestellt.
        Eine Verbindungsfehler-Ausnahme wird in einem der folgenden Faelle erzeugt:
        - Ein Objekt ist bereits verbunden
          (moeglich, falls die Methode nicht ueber das Objekt aufgerufen wurde)
        - Ein Objekt ist bereits zu einer anderen Gruppe verbunden
          (moeglich, falls die Methode nicht ueber das Objekt aufgerufen wurde)
        - Objekt stellt keine Methoden zur Verbindungskontrolle
          (verbindeMitObjekt//gibVerbundenesObjekt) zur Verfuegung.
        """
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        if debug: print str(self) + ": objektVerbinden " + str(neuesobjekt)
        try:
            if self.__ZugeordneteGrafik is not None:
                raise Verbindungsfehler("Es ist bereits ein Objekt mit dem Grafikzeichner verbunden.")
            con = neuesobjekt.gibVerbundenesObjekt()
            if (con is None):
                neuesobjekt.verbindeMitObjekt(self)
                # Verbindung wird auf Objektseite initialisiert
            elif (con==self):
                if (con == self.__ZugeordneteGrafik):
                    raise Verbindungsfehler("Objekt ist bereits verbunden.")
                else:
                    self.__ZugeordneteGrafik=neuesobjekt
                    return True
            else:
                raise Verbindungsfehler("Objekt ist anderweitig verbunden.")      
        except AttributeError:
            raise Verbindungsfehler("Objekt stellt keine Methode zur Verbindungskontrolle zur Verfuegung.")
    
    def objektTrennen(self,objekt):
        """
        Es wird das verbundene Objekt vom Grafikzeichner entfernt.
        Ein Verbindungsfehler wird in einem der folgenden Faelle erzeugt:
        - Das Objekt war nicht mit der Gruppe verbunden.
        - Das Objekt stellt keine Methoden zur Verbindungskontrolle
          (trenneVonObjekt//gibVerbundenesObjekt) zur Verfuegung.
        """
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        if debug: print str(self) + ": objektTrennen " + str(objekt)
        if objekt != self.__ZugeordneteGrafik:
            raise Verbindungsfehler("Objekt war nicht mit dieser Vektorgrafik verbunden.")
        try:
            con = objekt.gibVerbundenesObjekt()
            if con is not None:
                # eine Verbindung zu einer anderen Gruppe ist durch vorherige
                # Abfrage ausgeschlossen
                objekt.trenneVonObjekt()
                # Trennung auf Objektseite initialisiert
            else:
                self.__ZugeordneteGrafik = None
                return True
        except AttributeError:
            self.__ZugeordneteGrafik = None
            raise Verbindungsfehler("Das Objekt stellt keine Methoden zur Verbindungskontrolle zur Verfuegung.")
    
    def objektGeaendert(self,obj):
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.') # wird automatisiert von verbundenen Vektorgrafiken aufgerufen
        if self.__ZugeordneteGrafik == obj:
            if self.automatikModus:
                self.__bildschirm.loescheAlles()
                self.zeichneGrafik(obj)
    
    def gibVerbundenesObjekt(self):
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        return self.__ZugeordneteGrafik
    
    def zeichneGrafik(self,vg):
        """
        Zeichnet die Grafikobjekte, die im gegebenen Vektorgrafikobjekt enthalten sind.
        Die Erkennung der Objekte funktioniert anhand des Klassennamen.
        Kann ein Objekt nicht erkannt werden, so wird versucht, die
        'zeichneGrafik'-Methode rekursiv aufzurufen.
        Ein Objekt von einer beliebigen Klasse kann also dann gezeichnet werden,
        wenn mit Hilfe der Methode 'gibObjektliste' eine Liste von
        Grafikprimitiven erhalten werden kann.
        """
        if not self.__freigegeben:
            raise Exception('Grafikzeichner ist nicht mehr verfuegbar.')
        if debug: print "Zeichne Grafik!!"
        objektliste = vg.gibObjektliste()
        regex = r"<class '(\w*\.\w*)'>"
        recomp = re.compile(regex)
        for obj in objektliste:
            if debug: print "Zeichne Objekt", obj
            clsstr = str(type(obj))
            m = recomp.match(clsstr)
            if m is not None:
                clsname = m.group(1)
                if clsname=="pyobjvg.Rechteck":
                    eckpunkte = obj.gibEckpunkte()
                    eckpunkte = [(pkt.x,pkt.y) for pkt in eckpunkte]
                    ffarbe = obj.fuellfarbe
                    if ffarbe is not None:
                        ffarbe = ffarbe.gibSUMFarbe()
                        self.__bildschirm.bild.drawpolygon(eckpunkte,ffarbe,0)
                    lfarbe = obj.linienfarbe
                    if lfarbe is not None:
                        lfarbe = lfarbe.gibSUMFarbe()
                    lstaerke = obj.linienstaerke
                    if (lstaerke > 0) and (lfarbe is not None):
                        self.__bildschirm.bild.drawpolygon(eckpunkte,lfarbe,lstaerke)
                elif clsname=="pyobjvg.Ellipse":
                    mpkt = obj.mittelpunkt.x,obj.mittelpunkt.y
                    ffarbe = obj.fuellfarbe
                    if ffarbe is not None:
                        ffarbe = ffarbe.gibSUMFarbe()
                        self.__bildschirm.bild.drawellipse(mpkt,obj.radiusx,obj.radiusy,ffarbe,0,obj.drehwinkel)
                    lfarbe = obj.linienfarbe
                    if lfarbe is not None:
                        lfarbe = lfarbe.gibSUMFarbe()
                    lstaerke = obj.linienstaerke
                    if lstaerke > 0 and lfarbe is not None:
                        self.__bildschirm.bild.drawellipse(mpkt,obj.radiusx,obj.radiusy,lfarbe,lstaerke,obj.drehwinkel)
                elif clsname=="pyobjvg.Kreis":
                    mpkt = obj.mittelpunkt.x,obj.mittelpunkt.y
                    ffarbe = obj.fuellfarbe
                    if ffarbe is not None:
                        ffarbe = ffarbe.gibSUMFarbe()
                        self.__bildschirm.bild.drawkreis(mpkt,obj.radius,ffarbe,0)
                    lfarbe = obj.linienfarbe
                    if lfarbe is not None:
                        lfarbe = lfarbe.gibSUMFarbe()
                    lstaerke = obj.linienstaerke
                    if lstaerke > 0 and lfarbe is not None:
                        self.__bildschirm.bild.drawkreis(mpkt,obj.radius,lfarbe,lstaerke)
                elif clsname=="pyobjvg.Polygonzug":
                    pktliste = obj.gibPunkteliste()
                    if pktliste != [ ]:
                        pktliste = [(pkt.x,pkt.y) for pkt in pktliste]
                        lfarbe = obj.linienfarbe
                        if lfarbe is not None:
                            lfarbe = lfarbe.gibSUMFarbe()
                        lstaerke = obj.linienstaerke
                        if lstaerke > 0 and lfarbe is not None:
                            self.__bildschirm.bild.drawpolyline(pktliste,lfarbe,lstaerke)
                elif clsname=="pyobjvg.Polygon":
                    pktliste = obj.gibPunkteliste()
                    if pktliste != [ ]:
                        pktliste = [(pkt.x,pkt.y) for pkt in pktliste]
                        lfarbe = obj.linienfarbe
                        if lfarbe is not None:
                            lfarbe = lfarbe.gibSUMFarbe()
                        ffarbe = obj.fuellfarbe
                        if ffarbe is not None:
                            ffarbe = ffarbe.gibSUMFarbe()
                        lstaerke = obj.linienstaerke
                        if ffarbe is not None:
                            self.__bildschirm.bild.drawpolygon(pktliste,ffarbe,0)
                        if lstaerke > 0 and lfarbe is not None:
                            self.__bildschirm.bild.drawpolygon(pktliste,lfarbe,lstaerke)
                elif clsname=="pyobjvg.Bezierkurve":
                    pktliste = obj.gibNaeherungspunkteliste()
                    if pktliste != [ ]:
                        pktliste = [(pkt.x,pkt.y) for pkt in pktliste]
                        lfarbe = obj.linienfarbe
                        if lfarbe is not None:
                            lfarbe = lfarbe.gibSUMFarbe()
                        lstaerke = obj.linienstaerke
                        if lstaerke > 0 and lfarbe is not None:
                            self.__bildschirm.bild.drawpolyline(pktliste,lfarbe,lstaerke)
                elif clsname=="pyobjvg.Linie":
                    pkte = obj.gibPunkte()
                    if pkte != [ ]:
                        anf,end = pkte
                        # Punkt2D fuer Pygame auspacken
                        anf = (anf.x,anf.y)
                        end = (end.x,end.y)
                        lfarbe = obj.linienfarbe
                        if lfarbe is not None:
                            lfarbe = lfarbe.gibSUMFarbe()
                            lstaerke = obj.linienstaerke
                            if lstaerke > 0 and lfarbe is not None:
                                self.__bildschirm.bild.drawpolyline([anf,end],lfarbe,lstaerke)
                elif clsname=="pyobjvg.Text":
                    text = obj.text
                    posx,posy = obj.mittelpunkt
                    lfarbe = obj.linienfarbe
                    ffarbe = obj.fuellfarbe
                    grad,drehwinkel = obj.schriftgrad,obj.drehwinkel
                    lstaerke = obj.linienstaerke
                    if lfarbe is not None:
                        lfarbe = lfarbe.gibSUMFarbe()
                    if ffarbe is not None:
                        ffarbe = ffarbe.gibSUMFarbe()
                    if grad > 0 and lfarbe is not None:
                        self.__bildschirm.bild.drawtext(text,(posx,posy),lfarbe,"",grad,drehwinkel)
                    # TODO: Realisierung gefuellter Texte fehlt noch,
                    # m. E. sinnvoll den Bildserver zu erweitern, mal sehen...
                else:
                    # konnte nicht als Grafikprimitiv erkannt
                    # werde, daher wird versucht, die
                    # Zeichnemethode rekursiv wieder aufzurufen.
                    try:
                        self.zeichneGrafik(obj)
                    except AttributeError:
                        pass
            else:
                # konnte nicht als Grafikprimitiv erkannt
                # werde, daher wird versucht, die
                # Zeichnemethode rekursiv wieder aufzurufen.
                try:
                    self.zeichneGrafik(obj)
                except AttributeError:
                    pass
    

    